@@ -1,5 +1,5 @@
 package HTML::Selector::Element;
-our $VERSION = '0.95_03';
+our $VERSION = '0.95';
 
 ## Adapted from HTML::Selector::XPath
 ## The parser is basically the same, the difference is in what it produces.
@@ -11,8 +11,7 @@ sub import {
     my($class) = shift;
     if(@_) {
         require HTML::Element;
-        package # hide from pause
-            HTML::Element;
+        package HTML::Element;
         local $^W;  # no warnings 'redefined' doesn't work over there in Exporter
         HTML::Selector::Element::Trait->import(@_);           
     }
@@ -290,110 +289,110 @@ sub consume {
     return \@alt, $rule; 
 }
 
-sub criteria {
+sub criteria {     
     # returns criteria for look_down, with bound closures
     my($set, $refroot, $strategy) = @_;
     $strategy ||= { banroot => 1 };
-    $strategy->{banroot} = 0 if not $refroot;
     my $recurse;
-    $recurse = sub {
-        # embeds $refroot
-        my($set, $banroot) = @_;
-        my @params = @{$set->{static}||[]};
+    for my $root ($refroot ? $$refroot : 0) {
+        $recurse = sub {
+            # embeds $root
+            my($set, $banroot) = @_;
+            my @params = @{$set->{static}||[]};
 
-        if($set->{is_root}) {
-            $banroot = 0;
-            if($refroot) {
-                # relative, top of branch
-                unshift @params, sub { $_[0] == $$refroot };  # unlikely to succeed, so fail fast
-            }
-            else {
-                # absolute, root of DOM
-                unshift @params, _parent => undef;  # unlikely to succeed, so fail fast
-            }
-        }
-
-        if($set->{chained}) {
-            push @params, do {
-                # Value is an anonymous sub
-                # Recurse into linked list
-                my @params = $recurse->($set->{chained}, $banroot);
-                my $combinator = $set->{combinator};
-                # we're in a chained set, so we have to wrap the criteria into a test in a closure.
-                if ($combinator =~ /^\s+$/) {
-                    $banroot ? sub { my($this) = @_;
-                        my $rootparent = $$refroot->{_parent} || 0;
-                        while($this = $this->{_parent} and $this != $rootparent) {
-                            look_self($this, @params) 
-                                and return 1;
-                        }
-                        return 0;
-                    }
-                    : sub { my($this) = @_;
-                        while($this = $this->{_parent}) {
-                            look_self($this, @params) 
-                                and return 1;
-                        }
-                        return 0;
-                    }
-                }
-                elsif ($combinator =~ />/) {
-                    $banroot ? sub { my($this) = @_;
-                        my $rootparent = $$refroot->{_parent} || 0;
-                        if($this = $this->{_parent} and $this != $rootparent) {
-                            look_self($this, @params) 
-                                and return 1;
-                        }
-                        return 0;
-                    }
-                    : sub { my($this) = @_;
-                        if($this = $this->{_parent}) {
-                            look_self($this, @params) 
-                                and return 1;
-                        }
-                        return 0;
-                    };
-                }
-                elsif ($combinator =~ /\+/) {
-                    sub { my($this) = @_;
-                        my @left = $this->left;
-                        while(@left) {
-                            $this = pop @left;
-                            ref $this && $this->{_tag} && $this->{_tag} !~ /^~/ or next;
-                            look_self($this, @params) 
-                                and return 1;
-                            return 0;
-                        }
-                        return 0;
-                    };
-                }
-                elsif ($combinator =~ /\~/) {
-                    sub { my($this) = @_;
-                        my @left = $this->left;
-                        while(@left) {
-                            $this = pop @left;
-                            ref $this && $this->{_tag} !~ /^~/ or next;
-                            look_self($this, @params) 
-                                and return 1;
-                        }
-                        return 0;
-                    }
+            if($set->{is_root}) {
+                $banroot = 0;
+                if($refroot) {
+                    # relative, top of branch
+                    unshift @params, sub { $_[0] == $root };  # unlikely to succeed, so fail fast
                 }
                 else {
-                    die "Weird combinator '$combinator'"
+                    # absolute, root of DOM
+                    unshift @params, _parent => undef;  # unlikely to succeed, so fail fast
                 }
-            };
-        }
-        elsif($banroot && !$set->{is_root}) {
-            # if :root was not specified, $root should never match
-            push @params, sub { $_[0] != $$refroot };   # likely to succeed, so fail late
-        }
+            }
 
-        # do the :has tests last, because it's a complete subtree scan and that may be very slow.
-        push @params, @{$set->{has}} if $set->{has};
-        return wantarray ? @params : \@params;
-    };
+            if($set->{chained}) {
+                push @params, do {
+                    # Value is an anonymous sub
+                    # Recurse into linked list
+                    my @params = $recurse->($set->{chained}, $banroot);
+                    my $combinator = $set->{combinator};
+                    # we're in a chained set, so we have to wrap the criteria into a test in a closure.
+                    if ($combinator =~ /^\s+$/) {
+                        $banroot ? sub { my($this) = @_;
+                            my $rootparent = $root->{_parent} || 0;
+                            while($this = $this->{_parent} and $this != $rootparent) {
+                                look_self($this, @params) 
+                                    and return 1;
+                            }
+                            return 0;
+                        }
+                        : sub { my($this) = @_;
+                            while($this = $this->{_parent}) {
+                                look_self($this, @params) 
+                                    and return 1;
+                            }
+                            return 0;
+                        }
+                    }
+                    elsif ($combinator =~ />/) {
+                        $banroot ? sub { my($this) = @_;
+                            my $rootparent = $root->{_parent} || 0;
+                            if($this = $this->{_parent} and $this != $rootparent) {
+                                look_self($this, @params) 
+                                    and return 1;
+                            }
+                            return 0;
+                        }
+                        : sub { my($this) = @_;
+                            if($this = $this->{_parent}) {
+                                look_self($this, @params) 
+                                    and return 1;
+                            }
+                            return 0;
+                        };
+                    }
+                    elsif ($combinator =~ /\+/) {
+                        sub { my($this) = @_;
+                            my @left = $this->left;
+                            while(@left) {
+                                $this = pop @left;
+                                ref $this && $this->{_tag} && $this->{_tag} !~ /^~/ or next;
+                                look_self($this, @params) 
+                                    and return 1;
+                                return 0;
+                            }
+                            return 0;
+                        };
+                    }
+                    elsif ($combinator =~ /\~/) {
+                        sub { my($this) = @_;
+                            my @left = $this->left;
+                            while(@left) {
+                                $this = pop @left;
+                                ref $this && $this->{_tag} !~ /^~/ or next;
+                                look_self($this, @params) 
+                                    and return 1;
+                            }
+                            return 0;
+                        }
+                    }
+                    else {
+                        die "Weird combinator '$combinator'"
+                    }
+                };
+            }
+            elsif($banroot && !$set->{is_root}) {
+                # if :root was not specified, $root should never match
+                push @params, sub { $_[0] != $root };   # likely to succeed, so fail late
+            }
 
+            # do the :has tests last, because it's a complete subtree scan and that may be very slow.
+            push @params, @{$set->{has}} if $set->{has};
+            return wantarray ? @params : \@params;
+        };
+    }
 
     if(ref $set eq 'ARRAY') {
         if(@$set > 1) {
